<?php

namespace Realestate\Video\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Realestate\Video\Forms\Video as VideoForm;
use Realestate\Video\Http\Requests\VideoResourceRequest;
use Realestate\Video\Http\Resources\VideoResource;
use Realestate\Video\Http\Resources\VideosCollection;
use Realestate\Video\Models\Video;
use Realestate\Video\Actions\VideoAction;
use Realestate\Video\Actions\VideoActions;


/**
 * Resource controller class for video.
 */
class VideoResourceController extends BaseController
{

    /**
     * Initialize video resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = VideoForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('realestate.video.modules'), 'video', guard_url('video'));
            return $next($request);
        });
    }

    /**
     * Display a list of video.
     *
     * @return Response
     */
    public function index(VideoResourceRequest $request)
    {
        $request = $request->all();
        $page = VideoActions::run('paginate', $request);

        $data = new VideosCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('video::video.names'))
            ->view('video::video.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display video.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function show(VideoResourceRequest $request, Video $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VideoResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('video::video.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('video::video.show')
            ->output();
    }

    /**
     * Show the form for creating a new video.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VideoResourceRequest $request, Video $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VideoResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('video::video.name'))
            ->view('video::video.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new video.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VideoResourceRequest $request, Video $model)
    {
        try {
            $request = $request->all();
            $model = VideoAction::run('store', $model, $request);
            $data = new VideoResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('video::video.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('video/video/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/video/video'))
                ->redirect();
        }

    }

    /**
     * Show video for editing.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function edit(VideoResourceRequest $request, Video $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VideoResource($model);
        // return view('video::video.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('video::video.name'))
            ->view('video::video.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the video.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function update(VideoResourceRequest $request, Video $model)
    {
        try {
            $request = $request->all();
            $model = VideoAction::run('update', $model, $request);
            $data = new VideoResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('video::video.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('video/video/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('video/video/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the video.
     *
     * @param Model   $video
     *
     * @return Response
     */
    public function destroy(VideoResourceRequest $request, Video $model)
    {
        try {

            $request = $request->all();
            $model = VideoAction::run('destroy', $model, $request);
            $data = new VideoResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('video::video.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('video/video/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('video/video/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
